/*
** Module   : ColorMapper.java
** Abstract : map P2J color specifications to fonts that can be used for CHUI 
**
** Copyright (c) 2011-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 GES 20110921 Created initial version.
** 002 MAG 20140211 Implements COLOR. 
** 002 MAG 20140219 Fixed NPE when color is null. 
*/

package com.goldencode.p2j.ui.client.swing;

import java.awt.*;
import java.awt.font.*;
import java.util.*;

// we import selectively because there are many overlapping class names between
// swing and p2j, since the current class is more swing than p2j we err on the
// side of swing imports
import com.goldencode.p2j.ui.Color;
import com.goldencode.p2j.ui.ColorPalette;

/**
 * Converts Progress-compatible color specifications (see
 * {@link com.goldencode.p2j.ui.Color}) into a fixed width font which can be
 * used to display that text in a CHUI which uses that color specification.
 * <p>
 * At this time, only the attributes member of a color specification is
 * honored.  The color member is not supported.
 * <p>
 * The <code>BLINK</code> attribute cannot be supported directly with a font
 * alone (all other attributes can be duplicated with a single font).  Instead
 * the calling code will have to use a timer to periodically redraw the font
 * while inverting the colors each time.
 */
public class ColorMapper
{
   /** Cache of fonts mapped to the color instances they represent. */
   private Map<Color, Font> map = null;
   
   /**
    * Constructor.
    *
    * @param    font
    *           The base font which defines <code>Color.NORMAL</code> and from
    *           which all other "colors" are derived.
    */
   public ColorMapper(Font font)
   {
      reset(font);
   }
   
   /**
    * Reset the cache based on the given font. This is a no-operation if the
    * given font is the same as the current base font.
    *
    * @param    font
    *           The base font which defines <code>Color.NORMAL</code> and from
    *           which all other "colors" are derived.
    */
   public void reset(Font font)
   {
      if (map == null)
      {
         // first initialization
         map = new HashMap<Color, Font>();
      }
      else
      {
         Font old = map.get(Color.NORMAL);
         
         if (!old.equals(font))
         {
            // reset
            map.clear();
         }
         else
         {
            // bypass the map insert below
            font = null;
         }
      }
      
      if (font != null)
         map.put(Color.NORMAL, font);
   }
   
   /**
    * Derive a font from <code>Color.NORMAL</code> which has the attributes
    * specified in the given color AND then return the reverse of that font.
    * parameter is not honored, only the attribute member is handled.
    * <p>
    * <code>BLINK</code> is not supported at this time.
    *
    * @param    color
    *           The color specification to convert into a font.
    *
    * @return   The font that matches the reverse of the color specification.
    */
   public Font toFontReverse(Color color)
   {
      Font normal = toFont(color);
      
      Map<TextAttribute, Object> attr = new HashMap<TextAttribute, Object>();
      attr.put(TextAttribute.SWAP_COLORS, TextAttribute.SWAP_COLORS_ON);
         
      return normal.deriveFont(attr);
   }
   
   /**
    * Derive a font from <code>Color.NORMAL</code> which has the attributes
    * specified in the given color. If that font has already been derived, it
    * will be returned from a cache. At this time, the color member of the
    * parameter is not honored, only the attribute member is handled.
    * <p>
    * <code>BLINK</code> is not supported at this time.
    *
    * @param    color
    *           The color specification to convert into a font.
    *
    * @return   The font that matches the color specification.
    */
   public Font toFont(Color color)
   {
      Font font = map.get(color);
      
      if (font == null)
      {
         Map<TextAttribute, Object> attr = new HashMap<TextAttribute, Object>();
         
         if (color != null)
         {
            // FG color.
            ColorPalette foreground = Color.toForeground(color);                  
            if (foreground != null)
            {
               attr.put(TextAttribute.FOREGROUND, foreground.getColor());
            }
   
            // BG color.
            ColorPalette background = Color.toBackground(color);
            if (background != null)
            {
               attr.put(TextAttribute.BACKGROUND, background.getColor());
            }                  
            
            if (color.hasUnderline())
            {
               attr.put(TextAttribute.UNDERLINE, TextAttribute.UNDERLINE_ON);
            }
            
            if (color.hasReverse())
            {
               attr.put(TextAttribute.SWAP_COLORS, TextAttribute.SWAP_COLORS_ON);
            }
            
            if (color.hasBold())
            {
               attr.put(TextAttribute.WEIGHT, TextAttribute.WEIGHT_BOLD);
            }
            
            if (color.hasBlink())
            {
               // TODO: needs a timer-based implementation, can't be implemented
               //       only with a font!
            }
         }
                           
         // derive the new font
         font = map.get(Color.NORMAL).deriveFont(attr);
        
         // cache it
         map.put(color, font);
      }
      
      return font;
   }
}
